from nmdc_schema.migrators.migrator_base import MigratorBase


class Migrator(MigratorBase):
    """Migrates data from schema X to PR4"""

    _from_version = "X"
    _to_version = "PR4"

    def upgrade(self):
        r"""Migrates the database from conforming to the original schema, to conforming to the new schema."""

        self.adapter.process_each_document(collection_name="omics_processing_set",
                                           pipeline=[self.update_analyte_category_from_omics_type])

    def update_analyte_category_from_omics_type(self, omics_proc: dict):
        r"""
        Transforms values in the 'omics_type.has_raw_value' slot to match the accepted AnalyteCategoryEnum and adds
        a new slot called analyte_category with the accepted value. Removes the omics_type slot.
        
        >>> m = Migrator()
        >>> m.update_analyte_category_from_omics_type({'id': 123, 'omics_type': {'has_raw_value': 'Organic Matter Characterization'}})
        {'id': 123, 'analyte_category': 'nom'}
        """

        # Maps all the existing values for omics_type.has_raw_value to the allowed values for 
        # analyte category. Used Mongodb distinct in a separate query to get all the existing values.
        omics_type_to_analyte_category_mapping = {"Metagenome": "metagenome",
                                                  "metagenome": "metagenome",
                                                  "Metatranscriptome": "metatranscriptome",
                                                  "Proteomics": "metaproteome",
                                                  "Metabolomics": "metabolome",
                                                  "Lipidomics": "lipidome",
                                                  "Organic Matter Characterization": "nom"
                                              }
        
        omics_type = omics_proc.get("omics_type").get("has_raw_value")

        # If the value for omics type is a key in the mapping dictionary, then create a slot
        # analyte_cateogry with the value, the value from the mapping dictionary.
        if omics_type in omics_type_to_analyte_category_mapping:
            omics_proc["analyte_category"] = omics_type_to_analyte_category_mapping[omics_type]

            # remove omics_type slot
            del omics_proc["omics_type"]
    
        else:
            raise ValueError(f"The 'omics_type' value ({omics_type}) of document '{omics_proc['id']}' "
                             f"does not match any 'analyte_category' values.")

        return omics_proc

    


from typing import Optional, Callable, List, Union
from nmdc_schema.migrators.adapters.adapter_base import AdapterBase
from pymongo.database import Database


class MongoAdapter(AdapterBase):
    r"""
    Class containing methods related to manipulating a MongoDB database.
    """

    def __init__(self, database: Database, **kwargs) -> None:
        r"""
        Invokes the initialization method of the parent class, passing to it any specified keyword arguments.
        Also initializes the reference to the database this adapter instance will be used to manipulate.

        References:
        - https://pymongo.readthedocs.io/en/stable/examples/type_hints.html#typed-database
        """
        super().__init__(**kwargs)
        self._db = database

    def create_collection(self, collection_name: str) -> None:
        r"""
        Creates an empty collection having the specified name, if no collection by that name exists.
        Also invokes `self.on_collection_created`, if defined, passing to it the name of the collection.

        References:
        - https://pymongo.readthedocs.io/en/stable/api/pymongo/database.html#pymongo.database.Database.create_collection
        - https://docs.python.org/3/library/functions.html#callable
        """
        if collection_name not in self._db.list_collection_names():
            self._db.create_collection(name=collection_name)

            # If the relevant callback function exists, invoke it.
            if callable(self.on_collection_created):
                self.on_collection_created(collection_name)

    def rename_collection(self, current_name: str, new_name: str) -> None:
        r"""
        Renames the specified collection, if it exists, so that it has the specified new name.
        Also invokes `self.on_collection_renamed`, if defined, passing to it the old and new names of the collection.

        References:
        - https://pymongo.readthedocs.io/en/stable/api/pymongo/database.html#pymongo.database.Database.get_collection
        - https://pymongo.readthedocs.io/en/stable/api/pymongo/collection.html#pymongo.collection.Collection.rename
        """
        if current_name in self._db.list_collection_names():
            self._db.get_collection(name=current_name).rename(new_name=new_name)

            # If the relevant callback function exists, invoke it.
            if callable(self.on_collection_renamed):
                self.on_collection_renamed(current_name, new_name)

    def delete_collection(self, collection_name: str) -> None:
        r"""
        Deletes the collection having the specified name, if such a collection exists.
        Also invokes `self.on_collection_deleted`, if defined, passing to it the name of the collection.

        References:
        - https://pymongo.readthedocs.io/en/stable/api/pymongo/database.html#pymongo.database.Database.drop_collection
        """
        if collection_name in self._db.list_collection_names():
            self._db.drop_collection(name_or_collection=collection_name)

            # If the relevant callback function exists, invoke it.
            if callable(self.on_collection_deleted):
                self.on_collection_deleted(collection_name)

    def insert_document(self, collection_name: str, document: dict) -> None:
        r"""
        Inserts the specified document into the collection having the specified name.

        References:
        - https://pymongo.readthedocs.io/en/stable/api/pymongo/collection.html#pymongo.collection.Collection.insert_one
        """
        self._db.get_collection(name=collection_name).insert_one(document)

    def get_document_having_value_in_field(
        self, collection_name: str, field_name: str, value: str
    ) -> Optional[dict]:
        r"""
        Retrieves the first document from the specified collection, having the specified value in the specified field.

        Note: This only supports top-level fields (e.g. `_id`, `depth`), not nested fields (e.g. `depth.has_unit`).

        References:
        - https://pymongo.readthedocs.io/en/stable/api/pymongo/collection.html#pymongo.collection.Collection.find_one
        - https://www.mongodb.com/docs/manual/reference/operator/query/eq/#syntax
        """

        # Create the filter for the query.
        # Note: I named this variable `filter_` because `filter` is a built-in Python function.
        filter_ = dict()
        filter_[field_name] = {"$eq": value}

        # Find and return the first matching document, if any.
        document = self._db.get_collection(name=collection_name).find_one(
            filter=filter_
        )
        return document

    def get_document_having_one_of_values_in_field(
        self, collection_name: str, field_name: str, values: List[str]
    ) -> Optional[dict]:
        r"""
        Retrieves the first document from the specified collection, having any one of the specified values in the
        specified field.

        Note: This only supports top-level fields (e.g. `_id`, `depth`), not nested fields (e.g. `depth.has_unit`).

        References:
        - https://www.mongodb.com/docs/manual/reference/operator/query/in/
        - https://www.mongodb.com/docs/manual/reference/operator/query/type/
        """

        # Create the filter for the query.
        #
        # Note: The `$in` operator has a special-case behavior where, if the field is an array, the operator
        #       looks for any matching element _within_ the array. That is not what I want in this situation.
        #       I want to only compare the value of the field, itself. Since `values` is a list of strings,
        #       I will only bother checking fields of type "string" (which excludes fields of type "array").
        #
        filter_ = dict()
        filter_[field_name] = {"$type": "string", "$in": values}

        # Find and return the first matching document, if any.
        document = self._db.get_collection(name=collection_name).find_one(
            filter=filter_
        )
        return document

    def delete_documents_having_value_in_field(
        self, collection_name: str, field_name: str, value: str
    ) -> int:
        r"""
        Deletes all documents from the specified collection, having the specified value in the specified field;
        and returns the number of documents that were deleted.

        Note: This only supports top-level fields (e.g. `_id`, `depth`), not nested fields (e.g. `depth.has_unit`).

        References:
        - https://pymongo.readthedocs.io/en/stable/api/pymongo/collection.html#pymongo.collection.Collection.delete_many
        """

        # Create the filter for the query.
        # Note: I named this variable `filter_` because `filter` is a built-in Python function.
        filter_ = dict()
        filter_[field_name] = {"$eq": value}

        # Delete all matching documents.
        result = self._db.get_collection(name=collection_name).delete_many(
            filter=filter_
        )

        # Return the number of documents that were deleted.
        num_deleted = result.deleted_count
        return num_deleted

    def process_each_document(
        self, collection_name: str, pipeline: List[Callable[[dict], dict]]
    ) -> None:
        r"""
        Passes each document in the specified collection through the specified processing pipeline—in which
        the output of any given function is the input to the function after it—and stores the final output
        back in the collection, replacing the original document.

        References:
        - https://docs.python.org/3/library/copy.html#copy.deepcopy
        - https://pymongo.readthedocs.io/en/stable/api/pymongo/collection.html#pymongo.collection.Collection.replace_one
        """

        # Iterate over every document in the collection, if the collection exists.
        if collection_name in self._db.list_collection_names():
            collection = self._db.get_collection(name=collection_name)
            for document in collection.find():
                # Create a filter based upon this document's `_id` value, so that we can find the same document later.
                # We do this up front in case a function in the pipeline "inadvertently" tampers with the `_id` field.
                document_id = document["_id"]
                filter_ = {"_id": {"$eq": document_id}}

                # "Pass" the document through the functions (i.e. "stages") that make up the pipeline,
                # such that the output from one stage becomes the input to the next stage.
                for function in pipeline:
                    document = function(document)

                # Overwrite the original document with the processed one.
                collection.replace_one(filter=filter_, replacement=document)

    def set_field_of_each_document(
        self, collection_name: str, field_name: str, value: Union[None, str, int, float, bool],
    ) -> None:
        r"""
        Assigns the specified value to the specified field of each document in the collection.
        This method is a specialized alternative to the `process_each_document` method.
        """

        # Update every document in the collection, if the collection exists.
        if collection_name in self._db.list_collection_names():
            collection = self._db.get_collection(name=collection_name)
            collection.update_many({}, {"$set": {field_name: value}})

    def remove_field_from_each_document(
        self,
        collection_name: str,
        field_name: str,
    ) -> None:
        r"""
        Removes the specified field from each document in the collection.

        References:
        - https://www.mongodb.com/docs/manual/reference/operator/update/unset/
        """

        # Iterate over every document in the collection, if the collection exists.
        if collection_name in self._db.list_collection_names():
            collection = self._db.get_collection(name=collection_name)
            collection.update_many({}, {"$unset": {field_name: 0}})  # value is arbitrary (e.g. 0)

    def do_for_each_document(
        self, collection_name: str, action: Callable[[dict], None]
    ) -> None:
        r"""
        Passes each document in the specified collection to the specified function. This method was designed
        to facilitate iterating over all documents in a collection without actually modifying them.
        """

        # Iterate over every document in the collection, if the collection exists.
        if collection_name in self._db.list_collection_names():
            collection = self._db.get_collection(name=collection_name)
            for document in collection.find():
                action(document)

    def copy_value_from_field_to_field_in_each_document(
        self,
        collection_name: str,
        source_field_name: str,
        destination_field_name: str,
    ) -> None:
        r"""
        For each document in the collection that has the source field, copy the value of the source field
        into the destination field, creating the destination field if it doesn't already exist.

        References:
        - https://www.mongodb.com/docs/manual/reference/method/db.collection.updateMany
        - https://www.mongodb.com/docs/manual/reference/operator/update/set/
        - https://pymongo.readthedocs.io/en/stable/api/pymongo/collection.html#pymongo.collection.Collection.update_many
        """

        # Update every document in the collection, if the collection exists.
        if collection_name in self._db.list_collection_names():
            collection = self._db.get_collection(name=collection_name)
            collection.update_many(
                {source_field_name: {"$exists": True}},
                [{"$set": {destination_field_name: f"${source_field_name}"}}]
            )



from nmdc_schema.migrators.migrator_base import MigratorBase
import re


class Migrator(MigratorBase):
    r"""
    Migrates a database between two schemas.

    This migrator removes the `has_calibration` field from all documents that represent an instance of
    the `NomAnalysis` and 'MetabolomicsAnalysis' class, and moves the information to its corresponding
    'MassSpectrometry` `has_calibration` slot.

    The creation of this migrator was in response to this issue:
    https://github.com/microbiomedata/nmdc-schema/issues/2139

    """

    _from_version = "11.0.3"
    _to_version = "11.1.0.part_1"

    def upgrade(self) -> None:
        r"""
        Migrates the database from conforming to the original schema, to conforming to the new schema.

        >>> from nmdc_schema.migrators.adapters.dictionary_adapter import DictionaryAdapter
        >>> db = {
        ...   'workflow_execution_set': [
        ...     {'id': 'nmdc:wfx1', 'has_calibration': 'nmdc:dobj-13-abc123', 'was_informed_by': 'nmdc:dgen1', 'type': 'nmdc:MetabolomicsAnalysis'},
        ...     {'id': 'nmdc:wfx2', 'has_calibration': 'false', 'was_informed_by': 'nmdc:dgen2', 'type': 'nmdc:NomAnalysis'},
        ...     {'id': 'nmdc:wfx3', 'was_informed_by': 'nmdc:dgen3', 'type': 'nmdc:MetabolomicsAnalysis'}
        ...   ],
        ...   'data_generation_set': [
        ...     {'id': 'nmdc:dgen1'},
        ...     {'id': 'nmdc:dgen2'},
        ...     {'id': 'nmdc:dgen3'}
        ...   ],
        ...   'data_object_set': [
        ...     {'id': 'nmdc:dobj-13-abc123'}
        ...   ],
        ...   'calibration_set': [
        ...     {'id': 'nmdc:calib1', 'calibration_object': 'nmdc:dobj-13-abc123'}
        ...   ]
        ... }
        >>> a = DictionaryAdapter(database=db)
        >>> m = Migrator(adapter=a)
        >>> m.upgrade()
        >>> any('has_calibration' in doc for doc in db['workflow_execution_set'])  # Calibrations removed from workflow
        False
        >>> db['data_generation_set'][0]  # Calibration moved to data generation
        {'id': 'nmdc:dgen1', 'has_calibration': 'nmdc:calib1'}
        >>> db['data_generation_set'][1]  # No calibration added when value was 'false'
        {'id': 'nmdc:dgen2'}
        """
       

        self.adapter.process_each_document(collection_name="workflow_execution_set", pipeline=[self.store_and_remove_calibrations])
        self.adapter.process_each_document(collection_name="data_generation_set", pipeline=[self.update_data_gen_calibration])

    def check_has_calibration(self, has_calibration_value) -> bool:
        r"""
        Checks for a valid data object id format (starts with 'nmdc:dobj')

        >>> from nmdc_schema.migrators.adapters.dictionary_adapter import DictionaryAdapter
        >>> db = {}
        >>> a = DictionaryAdapter(database=db)
        >>> m = Migrator(adapter=a)
        >>> m.check_has_calibration('nmdc:dobj-13-abc123')  # Valid format
        True
        >>> m.check_has_calibration('false')  # Invalid format
        False
        >>> m.check_has_calibration('nmdc:something-else')  # Invalid format
        False
        """

        pattern = r'^nmdc:dobj'

        return bool(re.match(pattern, has_calibration_value))
    
    def check_for_valid_data_object(self, data_obj_id) -> bool:
        r"""
        Checks database for valid data object. Returns False if not valid

        >>> from nmdc_schema.migrators.adapters.dictionary_adapter import DictionaryAdapter
        >>> db = {
        ...   'data_object_set': [
        ...     {'id': 'nmdc:dobj-13-abc123'},
        ...     {'id': 'nmdc:dobj-13-def456'}
        ...   ]
        ... }
        >>> a = DictionaryAdapter(database=db)
        >>> m = Migrator(adapter=a)
        >>> m.check_for_valid_data_object('nmdc:dobj-13-abc123')  # Exists in database
        True
        >>> m.check_for_valid_data_object('nmdc:dobj-13-nonexistent')  # Doesn't exist
        False
        """

        data_obj_doc = self.adapter.get_document_having_value_in_field(
                    collection_name="data_object_set", field_name="id", value=data_obj_id
                    )
        
        return data_obj_doc is not None

    def store_and_remove_calibrations(self, workflow_execution_doc) -> dict:
        r"""
        Moves the `has_calibration` field from the `WorkflowExecution` document to
        the corresponding `DataGeneration` document.

        >>> from nmdc_schema.migrators.adapters.dictionary_adapter import DictionaryAdapter
        >>> db = {
        ...   'workflow_execution_set': [
        ...     {'id': 'nmdc:wfx1', 'has_calibration': 'nmdc:dobj-13-abc123', 'was_informed_by': 'nmdc:dgen1'},
        ...     {'id': 'nmdc:wfx2', 'has_calibration': 'false', 'was_informed_by': 'nmdc:dgen2'}
        ...   ],
        ...   'data_generation_set': [
        ...     {'id': 'nmdc:dgen1'},
        ...     {'id': 'nmdc:dgen2'}
        ...   ],
        ...   'data_object_set': [
        ...     {'id': 'nmdc:dobj-13-abc123'}
        ...   ],
        ...   'calibration_set': [
        ...     {'id': 'nmdc:calib1', 'calibration_object': 'nmdc:dobj-13-abc123'}
        ...   ]
        ... }
        >>> a = DictionaryAdapter(database=db)
        >>> m = Migrator(adapter=a)
        >>> workflow_execution_doc = {'id': 'nmdc:wfx1', 'has_calibration': 'nmdc:dobj-13-abc123', 'was_informed_by': 'nmdc:dgen1'}
        >>> m.store_and_remove_calibrations(workflow_execution_doc)
        {'id': 'nmdc:wfx1', 'was_informed_by': 'nmdc:dgen1'}
        >>> workflow_execution_doc = {'id': 'nmdc:wfx2', 'has_calibration': 'false', 'was_informed_by': 'nmdc:dgen2'}
        >>> m.store_and_remove_calibrations(workflow_execution_doc)
        {'id': 'nmdc:wfx2', 'was_informed_by': 'nmdc:dgen2'}
        >>> workflow_execution_doc = {'id': 'nmdc:wfx3', 'has_calibration': 'invalid', 'was_informed_by': 'nmdc:dgen3'}
        >>> m.store_and_remove_calibrations(workflow_execution_doc)  # doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
        ValueError: The 'has_calibration' value (invalid) in document (nmdc:wfx3) is not recognized
        """

        calibration_mapping = {} #create dictionary to store mappings

        if "has_calibration" in workflow_execution_doc:
            has_calibration_data_obj_id = workflow_execution_doc.get("has_calibration")

            # If has_calibration has a string value of false, remove the slot altogether from the document
            if has_calibration_data_obj_id.lower() == 'false':
                workflow_execution_doc.pop("has_calibration")
            
            # If the has_calibration value is not a data object id or does not have a value of "false"
            # raise an error.
            elif not self.check_has_calibration(has_calibration_data_obj_id):
                raise ValueError(f"The 'has_calibration' value ({has_calibration_data_obj_id}) in document "
                             f"({workflow_execution_doc['id']}) is not recognized")
            
            # If has_calibration is a nmdc data object identifier:
            elif self.check_has_calibration(has_calibration_data_obj_id):
    
                if not self.check_for_valid_data_object(has_calibration_data_obj_id):
                    raise ValueError(f"The 'has_calibration' value ({has_calibration_data_obj_id}) in document "
                             f"({workflow_execution_doc['id']}) is not a valid data object. The data object does not exist")
                else:
                    data_gen_doc = self.adapter.get_document_having_value_in_field(
                        collection_name="data_generation_set", field_name="id", value=workflow_execution_doc["was_informed_by"])
                    
                    calibration_doc = self.adapter.get_document_having_value_in_field(
                        collection_name="calibration_set", field_name="calibration_object", value=has_calibration_data_obj_id)
                   
                   # Store has_calibrations in calibration_mapping dictionary
                    calibration_mapping[data_gen_doc["id"]] = calibration_doc["id"]

                    if not hasattr(self, "calibration_mappings"):
                        self.calibration_mappings = {}
                    self.calibration_mappings.update(calibration_mapping)

                    # Remove calibration slot after storing mappings
                    workflow_execution_doc.pop("has_calibration")

        return workflow_execution_doc
    
    def update_data_gen_calibration(self, data_gen_doc) -> dict:
        r"""
        Updates data generation documents with calibration information

        >>> from nmdc_schema.migrators.adapters.dictionary_adapter import DictionaryAdapter
        >>> db = {
        ...   'workflow_execution_set': [
        ...     {'id': 'nmdc:wfx1', 'has_calibration': 'nmdc:dobj-13-abc123', 'was_informed_by': 'nmdc:dgen1', 'type': 'nmdc:MetabolomicsAnalysis'}
        ...   ],
        ...   'data_generation_set': [
        ...     {'id': 'nmdc:dgen1'},
        ...     {'id': 'nmdc:dgen2'}  # doc without corresponding calibration
        ...   ],
        ...   'data_object_set': [
        ...     {'id': 'nmdc:dobj-13-abc123'}
        ...   ],
        ...   'calibration_set': [
        ...     {'id': 'nmdc:calib1', 'calibration_object': 'nmdc:dobj-13-abc123'}
        ...   ]
        ... }
        >>> a = DictionaryAdapter(database=db)
        >>> m = Migrator(adapter=a)
        >>> # First store calibrations
        >>> workflow_execution_doc = {'id': 'nmdc:wfx1', 'has_calibration': 'nmdc:dobj-13-abc123', 'was_informed_by': 'nmdc:dgen1', 'type': 'nmdc:MetabolomicsAnalysis'}
        >>> _ = m.store_and_remove_calibrations(workflow_execution_doc)  # Store the calibrations first
        >>> # Then test update_data_gen_calibration
        >>> m.update_data_gen_calibration({'id': 'nmdc:dgen1'})  # doc with corresponding calibration
        {'id': 'nmdc:dgen1', 'has_calibration': 'nmdc:calib1'}
        >>> # Test document without calibration
        >>> m.update_data_gen_calibration({'id': 'nmdc:dgen2'})  # doc without corresponding calibration
        {'id': 'nmdc:dgen2'}
        """

        if data_gen_doc["id"] in self.calibration_mappings:
            data_gen_doc["has_calibration"] = self.calibration_mappings[data_gen_doc["id"]]
        return data_gen_doc



import logging
from typing import Any
from importlib import resources
from pathlib import Path
import yaml

logger = logging.getLogger(__name__)

# TODO: Determine directory path and package path dynamically, instead of hard-coding them.
MIGRATOR_DIRECTORY = "nmdc_schema/migrators"
MIGRATOR_PACKAGE = "nmdc_schema.migrators"


def load_yaml_asset(path_to_asset_file: str) -> Any:
    """
    Safely loads the contents of a YAML file into a Python object. This function was designed specifically to
    work in both the `nmdc-schema` development environment and in environments that import the `nmdc-schema` package.

    Parameters:
        path_to_asset_file: Filesystem path to the YAML file, relative to the root of the `assets` directory.

    References:
    - https://docs.python.org/3/library/pathlib.html#pathlib.PurePath.joinpath
    - https://docs.python.org/3/library/importlib.resources.html#importlib.resources.files
    - https://docs.python.org/3/library/importlib.resources.html#importlib.resources.as_file
    - https://realpython.com/python-yaml/#load-a-document-from-a-string-a-file-or-a-stream
    """
    # Define the Python path someone could use to `import` the package containing the `assets` directory.
    package_import_path = MIGRATOR_PACKAGE

    # Define the filesystem path to the `assets` directory (relative to that package).
    path_to_assets_directory = "assets"

    # Build a filesystem path to the specified asset file (also relative to that package).
    path_to_asset = Path(path_to_assets_directory).joinpath(path_to_asset_file)

    # Create a `Traversable` object that can be passed to the `resources.as_file()` function.
    traversable_resource_container = resources.files(package_import_path).joinpath(path_to_asset)

    # Use `resources.as_file()` to get an `open()`-able path to the asset file.
    with resources.as_file(traversable_resource_container) as file_path:
        # Finally, open the asset file and parse its contents as YAML.
        with open(file_path, "r") as f:
            obj = yaml.safe_load(f)  # the object type depends upon the YAML data

    return obj



from nmdc_schema.migrators.migrator_base import MigratorBase


class Migrator(MigratorBase):
    r"""
    Migrates a database between two schemas.

    This migrator removes the `part_of` field from all documents that represent an instance of
    the `WorkflowExecution` class or any of its subclasses.

    The creation of this migrator was prompted by this GitHub comment:
    https://github.com/microbiomedata/berkeley-schema-fy24/pull/210#pullrequestreview-2119492107

    This migrator was designed to be run before `migrator_from_10_2_0_to_11_0_0_part_16.py`, since that one
    moves the documents from all of these collections into one named `workflow_execution_set`.
    """

    _from_version = "X"
    _to_version = "PR104"

    collection_names = [
        "workflow_execution_set",
        "metagenome_annotation_set",
        "metagenome_assembly_set",
        "metatranscriptome_assembly_set",
        "metatranscriptome_annotation_set",
        "metatranscriptome_analysis_set",
        "mags_set",
        "metagenome_sequencing_set",
        "read_qc_analysis_set",
        "read_based_taxonomy_analysis_set",
        "metabolomics_analysis_set",
        "metaproteomics_analysis_set",
        "nom_analysis_set",
    ]

    def upgrade(self) -> None:
        r"""
        Migrates the database from conforming to the original schema, to conforming to the new schema.

        >>> from nmdc_schema.migrators.adapters.dictionary_adapter import DictionaryAdapter
        >>> db = {
        ...   'workflow_execution_set': [
        ...     {'id': 1},
        ...     {'id': 2, 'part_of': 'a'},
        ...     {'id': 3, 'part_of': 'b', 'foo': 'bar'},
        ...   ],
        ...   'metagenome_annotation_set': [
        ...     {'id': 1},
        ...     {'id': 2, 'part_of': 'a'},
        ...     {'id': 3, 'part_of': 'b', 'foo': 'bar'},
        ...   ]
        ... }
        >>> any("part_of" in document for document in db["workflow_execution_set"])
        True
        >>> any("part_of" in document for document in db["metagenome_annotation_set"])
        True
        >>> a = DictionaryAdapter(database=db)
        >>> m = Migrator(adapter=a)
        >>> m.upgrade()
        >>> any("part_of" in document for document in db["workflow_execution_set"])
        False
        >>> any("part_of" in document for document in db["metagenome_annotation_set"])
        False
        """

        for collection_name in self.collection_names:
            self.logger.info(f"Processing collection: {collection_name}")
            self.adapter.process_each_document(
                collection_name=collection_name, pipeline=[self.remove_part_of_field]
            )

    def remove_part_of_field(self, workflow_execution) -> dict:
        r"""
        Removes the `part_of` field from the document.

        >>> m = Migrator()
        >>> m.remove_part_of_field({'id': 123, 'part_of': 'whole'})
        {'id': 123}
        >>> m.remove_part_of_field({'id': 123, 'other': 'potato'})
        {'id': 123, 'other': 'potato'}
        >>> m.remove_part_of_field({'id': 123, 'other': 'potato', 'part_of': 'whole'})
        {'id': 123, 'other': 'potato'}
        """

        if "part_of" in workflow_execution:
            del workflow_execution["part_of"]

        return workflow_execution
